#pragma once
#include <vector>
#include <cstdint>
#include <cmath>
#include "extern.h"
#include "imgui.h"
#include "audio/misc/circular_audio_buffer.h"

class audio_buffer;
class mixer_track;
class plugin_host;

inline float volume_bar_peak_duration = 1.0f;
inline float volume_bar_peak_decay = 0.2f;
inline constexpr float volume_bar_inf = 0.0001f;
inline mixer_track* selected_track = nullptr;

inline float mapping(float value, float in_min, float in_max, float out_min, float out_max) {
    return (value - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

inline sample_t gain_to_db(sample_t sample) {
    return 20.0f * log10f(sample);
}

inline float db_to_gain(sample_t db) {
    return powf(10.0f, db / 20.0f);
}

inline float db_to_percentage_log(float db_value, float min_db = -60.0f, float max_db = 0.0f) {
    if (db_value < min_db) db_value = min_db;
    if (db_value > max_db) db_value = max_db;
    // 计算对数比例
    float log_min = std::log10(1.0f + std::pow(10.0f, min_db / 20.0f));
    float log_max = std::log10(1.0f + std::pow(10.0f, max_db / 20.0f));
    float log_value = std::log10(1.0f + std::pow(10.0f, db_value / 20.0f));

    // 线性映射到0到100的范围
    float percentage = (log_value - log_min) / (log_max - log_min);
    return percentage;
}

inline float percentage_to_db(float percentage, float min_db = -60.0f, float max_db = 0.0f) {
    // 计算对数比例
    float log_min = std::log10(1.0f + std::pow(10.0f, min_db / 20.0f));
    float log_max = std::log10(1.0f + std::pow(10.0f, max_db / 20.0f));
    float log_value = log_min + percentage * (log_max - log_min);

    // 计算db值
    float db_value = 20.0f * std::log10(std::pow(10.0f, log_value) - 1.0f);
    return db_value;
}

bool vertical_volume_slider(const char* id, ImVec2 size, float* volume, float min_volume = 0.0f, float max_volume = 1.0f);

void draw_header();

void draw_instrument_track();
void draw_mixer();
void draw_mixer_track(mixer_track* track);
void draw_volume_bar(uint64_t uid, float height, circular_buffer_vector_type& in_buffer);
void draw_volume_bar(ImGuiID id, float height, const std::vector<float>& sample_value, const std::vector<float>& sample_peak);

void draw_mixer_track_effect_list(mixer_track* track);
void draw_selected_mixer_track_effect_list();
void draw_mixer_track_effect(plugin_host* effect);
